// ==============================================================
//                 ORBITER MODULE: BUGGYUMMU
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// BUGGYUMMU.cpp
// Control module for BUGGYUMMU vessel class
//

// ==============================================================
#define ORBITER_MODULE
#include "orbitersdk.h"
#include "BUGGYUMMU.h"
#include "OrbiterSoundSDK35.h"
VISHANDLE MainExternalMeshVisual = 0;



void BUGGYUMMU::clbkPostCreation (void)

{
BUGGYUMMUSoundID=ConnectToOrbiterSoundDLL3(GetHandle());
SoundOptionOnOff3(BUGGYUMMUSoundID,PLAYMAINTHRUST,FALSE);

SoundOptionOnOff3(BUGGYUMMUSoundID,PLAYHOVERTHRUST,FALSE);
SoundOptionOnOff3(BUGGYUMMUSoundID,PLAYATTITUDETHRUST,FALSE);

SoundOptionOnOff3(BUGGYUMMUSoundID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);

SoundOptionOnOff3(BUGGYUMMUSoundID,PLAYCABINAIRCONDITIONING,FALSE);

SoundOptionOnOff3(BUGGYUMMUSoundID,PLAYCABINRANDOMAMBIANCE,FALSE);

SoundOptionOnOff3(BUGGYUMMUSoundID,PLAYRADIOATC,FALSE);

SoundOptionOnOff3(BUGGYUMMUSoundID,DISPLAYTIMER,FALSE);
}// Constructor
BUGGYUMMU::BUGGYUMMU (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)


{

LIFT_SPEED =.25;
L_SPEED =.1;
t1 = 1;
i2l = 0.5;
i2r = 0.5;
i2=.5;
antenna_status = antenna_UP;
antenna_proc=0;

	DefineAnimations ();
	
}


void BUGGYUMMU::clbkSetClassCaps (FILEHANDLE cfg)
{
	// physical specs
	SetSize (2.3);
	SetEmptyMass (100.0);
	SetCW (0.3, 0.3, 0.6, 0.9);
	SetWingAspect (0.1);
	SetWingEffectiveness (0.1);
	SetCrossSections (_V(1.76,3.21,1.32));
	SetRotDrag (_V(0.1,0.1,0.1));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-4);
		SetBankMomentScale (1e-4);
	}
	SetPMI (_V(.43,.45,.25 ));
	SetTrimScale (0.05);
	SetCameraOffset (_V(0,.924,.095));
	
	SetTouchdownPoints  (_V(0,-.787,1), _V(-1.6,-.798,-1), _V(1.6,-.798,-1));; 

DOCKHANDLE main1=CreateDock (_V(0,1,0), _V(0,1,0), _V(0,0,-1));
ATTACHMENTHANDLE MAIN;
MAIN = CreateAttachment (true, _V(0,-.787,0),_V(0,1,0),_V(0,0,1),"BUG1",false);	
EnableTransponder (true);
EnableIDS (main1, true);
SetIDSChannel(main1,8);

    PROPELLANT_HANDLE TANK;
    TANK = CreatePropellantResource (FUELMASS);

	// ***************** thruster definitions *******************
	

	th_main = CreateThruster (_V(0,0,-4.3), _V(0,0,1), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
		
	th_retro = CreateThruster (_V(0,0,4.3), _V(0,0,-1), MAXRCSTH, TANK, ISP);
	CreateThrusterGroup (&th_retro, 1, THGROUP_RETRO);

	th_rcs[0] = CreateThruster (_V( 1,0, 1), _V(0,0,1),MAXRCSTH, TANK, ISP);
	//left front tread
	th_rcs[1] = CreateThruster (_V(-1,0, 1), _V(0,0,1), MAXRCSTH, TANK, ISP);
	// right front tread
	th_rcs[2] = CreateThruster (_V( 1,0,-1), _V(0,0,-1), MAXRCSTH, TANK, ISP);
	//left rear tread
	th_rcs[3] = CreateThruster (_V(-1,0,-1), _V(0,0,-1), MAXRCSTH, TANK, ISP);
    // right rear tread
	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[2];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	// ************************************************
	// Call clbkSetClassCaps_UMMU and select a member
	SelectedUmmuMember=0;
	clbkSetClassCaps_UMMu();	
	
// visual specs
// visual specs
BUG0 = AddMesh (oapiLoadMeshGlobal ("BUGGY1"));
SetMeshVisibilityMode (BUG0, MESHVIS_ALWAYS);

BUG1 = AddMesh (oapiLoadMeshGlobal ("UmmuBug1"));
SetMeshVisibilityMode (BUG1, MESHVIS_NEVER);

BUG2 = AddMesh (oapiLoadMeshGlobal ("UmmuBug1P"));
SetMeshVisibilityMode (BUG2, MESHVIS_NEVER);

BUG3 = AddMesh (oapiLoadMeshGlobal ("UmmuBug1SEC"));
SetMeshVisibilityMode (BUG3, MESHVIS_NEVER);

BUG4 = AddMesh (oapiLoadMeshGlobal ("UmmuBug1VIP"));
SetMeshVisibilityMode (BUG4, MESHVIS_NEVER);

BUG5 = AddMesh (oapiLoadMeshGlobal ("UmmuBug1MED"));
SetMeshVisibilityMode (BUG5, MESHVIS_NEVER);

BUG6 = AddMesh (oapiLoadMeshGlobal ("UmmuBug1SCI"));
SetMeshVisibilityMode (BUG6, MESHVIS_NEVER);

BUG7 = AddMesh (oapiLoadMeshGlobal ("UmmuBug1TECH"));
SetMeshVisibilityMode (BUG7, MESHVIS_NEVER);

BUG8 = AddMesh (oapiLoadMeshGlobal ("space1999manbug"));
SetMeshVisibilityMode (BUG8, MESHVIS_NEVER);


}
// =======================================================
// UMMU Class Caps
// =======================================================
void BUGGYUMMU::clbkSetClassCaps_UMMu(void)
{
	// InitUmmu
	UmmuSdKInitReturnCode=Crew.InitUmmu(GetHandle());	
//	int crewaboard=GetCrewTotalNumber();



	// GetUserUMmuVersion
	float UMmuVersion=Crew.GetUserUMmuVersion();

	// DefineAirLockShape
	Crew.DefineAirLockShape(TRUE,-2,2,-2,2,-2,2);			// Airlock open, 2 meter large 4 meter high 6 meter long (vessel local coordinate)

	// SetMembersPositionOrientationOnEVA
	Crew.SetMembersPosRotOnEVA(_V(-1.138,.371,-.171),_V(0,0,0));	// 3 meters in front of ship (z) facing direction of ship (vessel local coordinate)

	//maximum seat available in this ship
	Crew.SetMaxSeatAvailableInShip(1);

	//default crew member used when the ship is spawned by Orbiter's scenerio editor. 
	Crew.AddCrewMember("Paul Roper",41,65,74,"Capt");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Frank Craig",27,67,55,"Eng");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Craig Collins",25,70,45,"Ast");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Doug Jackson",45,70,45,"Doc" );	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Keith Ford",45,70,45,"Sec");	//(for name and id a-z A-Z 0-9 characters only)
	Crew.AddCrewMember("Paul Foster",45,70,45,"Sci" );	//(for name and id a-z A-Z 0-9 characters only)
//change mesh
	
	// see function "SendFocusScreenMessage"
	strcpy(cOurMessage,"");
	//change mesh
	//Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");
}

void BUGGYUMMU::clbkPostStep(double simt, double simdt, double mjd)
{
	//---------------------------------------------------------------------------
	// ProcessUniversalMMu
	int ReturnCode=Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
		case UMMU_TRANSFERED_TO_OUR_SHIP: 
			sprintf(SendFocusScreenMessage(),"%s \"%s\" aged %i was transfered to our ship",Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
			break;
		case UMMU_RETURNED_TO_OUR_SHIP:
			sprintf(SendFocusScreenMessage(),"%s \"%s\" aged %i entered into our ship",Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
			break;
	}

	//----------------------------------------------------------------------------
	// SetCrewMemberPulseBySlotNumber
	if(GroundContact()==TRUE)
	{
		// we check vertical speed
		int I;
		VECTOR3 vHorizonAirspeedVector={0};
		GetHorizonAirspeedVector (vHorizonAirspeedVector);
		double VertSpeed		=vHorizonAirspeedVector.y;
		if(VertSpeed<-3)
		{
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all :(
			for(I=0;I<Crew.GetCrewTotalNumber();I++)
			{
				Crew.SetCrewMemberPulseBySlotNumber(I,0);	// set cardiac pulse to zero
			}
			strcpy(SendFocusScreenMessage(),"Oooh no ! Crash - All crew aboard killed");
		}
	}

	//----------------------------------------------------------------------------
	// see function "SendFocusScreenMessage" below for explaination.
	if(GetHandle()==oapiGetFocusObject())
	{
		strcpy(oapiDebugString(),cOurMessage);
	}

double vsAirSpeed = GetAirspeed();	
i3 = GetControlSurfaceLevel(AIRCTRL_RUDDER);

VECTOR3 speedvec; // first we define our vector which will recieve the data
oapiGetFocusShipAirspeedVector(&speedvec);//then we retrieve its x,y,z

if (GroundContact()==true && GetAirspeed() >8 && (speedvec.z>0))
{AddForce(_V(0,0,-9e3),_V(0,0,0));}

if (GroundContact()==true && GetAirspeed() >8 && (speedvec.z<0))
{AddForce(_V(0,0,9e3),_V(0,0,0));}

double da = simdt * LIFT_SPEED;

const char *p1MiscID = Crew.GetCrewMiscIdBySlotNumber(0);

if(Crew.GetCrewTotalNumber()==0){

SetMeshVisibilityMode( BUG1, MESHVIS_NEVER ); 
SetMeshVisibilityMode( BUG2, MESHVIS_NEVER ); 
SetMeshVisibilityMode( BUG3, MESHVIS_NEVER ); 
SetMeshVisibilityMode( BUG4, MESHVIS_NEVER ); 
SetMeshVisibilityMode( BUG5, MESHVIS_NEVER ); 
SetMeshVisibilityMode( BUG6, MESHVIS_NEVER ); 
SetMeshVisibilityMode( BUG7, MESHVIS_NEVER ); 
SetMeshVisibilityMode( BUG8, MESHVIS_NEVER ); 
}


if(Crew.GetCrewTotalNumber()>0){
if ((strcmp(p1MiscID, "") == 0)|| (strcmp(p1MiscID, "Ast") == 0))
{
SetMeshVisibilityMode( BUG1, MESHVIS_ALWAYS ); 
}
else  SetMeshVisibilityMode( BUG1, MESHVIS_NEVER ); 	// return: Empty string on error or MiscID


if (strcmp(p1MiscID, "Capt") == 0)
{
SetMeshVisibilityMode( BUG2, MESHVIS_ALWAYS ); 
}
else  SetMeshVisibilityMode( BUG2, MESHVIS_NEVER ); 


if (strcmp(p1MiscID, "Sec") == 0)
{
SetMeshVisibilityMode( BUG3, MESHVIS_ALWAYS ); 
}
else  SetMeshVisibilityMode( BUG3, MESHVIS_NEVER ); 

if (strcmp(p1MiscID, "Vip") == 0)
{
SetMeshVisibilityMode( BUG4, MESHVIS_ALWAYS ); 
}
else  SetMeshVisibilityMode( BUG4, MESHVIS_NEVER ); 

if ((strcmp(p1MiscID, "Sci") == 0)||(strcmp(p1MiscID, "Bio") == 0)||(strcmp(p1MiscID, "Geo") == 0)||(strcmp(p1MiscID, "Exo") == 0)||(strcmp(p1MiscID, "Clim") == 0)||(strcmp(p1MiscID, "APhy") == 0)||(strcmp(p1MiscID, "Chem") == 0))

{
SetMeshVisibilityMode( BUG6, MESHVIS_ALWAYS ); 
}
else  SetMeshVisibilityMode( BUG6, MESHVIS_NEVER ); 
if ((strcmp(p1MiscID, "Med") == 0)||(strcmp(p1MiscID, "Doc") == 0))

{
SetMeshVisibilityMode( BUG5, MESHVIS_ALWAYS ); 
}
else  SetMeshVisibilityMode( BUG5, MESHVIS_NEVER ); 


if ((strcmp(p1MiscID, "Eng") == 0)||(strcmp(p1MiscID, "Min") == 0)||(strcmp(p1MiscID, "Tech") == 0)||(strcmp(p1MiscID, "Spe") == 0))

{
SetMeshVisibilityMode( BUG7, MESHVIS_ALWAYS ); 
}
else  SetMeshVisibilityMode( BUG7, MESHVIS_NEVER );

if (strcmp(p1MiscID, "SP99") == 0)
{
SetMeshVisibilityMode( BUG8, MESHVIS_ALWAYS ); 
}
else  SetMeshVisibilityMode( BUG8, MESHVIS_NEVER ); 



}



//Anim Wheels Forward
 
 if (GetThrusterLevel (th_main) >0 && i3 == 0)
 {
    if (i > 1)i=0;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i > 1)i=0;
    i = i + (vsAirSpeed /100);
     
    if (vsAirSpeed ==0) i=0;
    
  // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
    main_on = 1;
    retro_on = 0;
}

}

//Anim Wheels Backward

if (GetThrusterLevel (th_retro) >0 && i3 == 0)

{
    
    if (i < 0)i=1;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (vsAirSpeed /100);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
     main_on = 0;
    retro_on = 1;
    
}


}


//Anim Wheels Forward and Backward when moving without power

 //Anim Wheels Forward and Backward when moving without power

if (GetThrusterLevel (th_main)  == 0 && GetThrusterLevel (th_retro) == 0 && vsAirSpeed > 0.15 && i3 == 0)

{
    
	
	if (main_on == 0 && retro_on == 1)
	
	{
    if (i < 0)i=1;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (vsAirSpeed /100);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
     
    
}
}

if (main_on == 1 && retro_on == 0)
	
	{
    if (i > 1)i=0;
    
    if (vsAirSpeed > 0.15) {
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i >  1)i=0;
    i = i + (vsAirSpeed /100);
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i );
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleft, i);
    
    
    
    
}
}


}




//Anim Wheels Opposite Forward and Backward when turning

// Left turn
if (i3 < -0.15)

{
    
    if (i < 0)i=1;
    
    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (0.01 + (vsAirSpeed /100));
     
    if (vsAirSpeed ==0) i=1;
    
   // sprintf(oapiDebugString(),"anim %2.2f", i2 );
    
	SetAnimation (anim_wheelrightrev, i);
    SetAnimation (anim_wheelleft, i);
    
  
}

// Right turn
if (i3 > 0.15)

{
    
    if (i < 0)i=1;
    
    
	    
	    if (vsAirSpeed > 55) vsAirSpeed = 55;

    
    if (i < 0)i=1;
    i = i - (0.01 + (vsAirSpeed /100));
     
    if (vsAirSpeed ==0) i=1;
    
 
    
	SetAnimation (anim_wheelright, i);
    SetAnimation (anim_wheelleftrev, i);
    
   
    
}

    
    //Anim Levers
    //sprintf(oapiDebugString(),"anim %2.2f", ignition );
    // Forward Lever animation
    
    if (GetThrusterLevel (th_main) >0 && i3 == 0)
   //Forward Thrust applied not turning, move levers back 
    {
	    i2=i2 - 0.01;
	    if (i2 < .25) i2=.25;
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
	
    
       
	    if (GetThrusterLevel (th_main) == 0 && i2 < 0.5 && i3 == 0)
    
    {
	    i2=i2 + 0.01;
	    if (i2 > 0.5) i2= 0.5;
	    // no thrust, return levers to neutral
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    
    if (GetThrusterLevel (th_main) == 0 && GetThrusterLevel (th_retro) ==0 && i2 == 0.5 && i3 == 0)
    
    {
	    i2 = 0.5;
	   
	  //no thrust, lever neutral  
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    // Reverse Lever animation
    
    if (GetThrusterLevel (th_retro) >0 && i3 == 0 )
    
    {
	    i2=i2 + 0.01;
	    if (i2 > .75) i2= .75;
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
	
    //retro thrust move lever forward
       
	    if (GetThrusterLevel (th_retro) == 0 && i2 > 0.5 && i3 == 0)
    
    {
	    i2=i2 - 0.01;
	    if (i2 < 0.5) i2= 0.5;
	    
	 //no retro thrust return levers to neutral
	    
	    SetAnimation (anim_leverleft, i2);
	    SetAnimation (anim_leverright, i2);
	    
    }
    
    //Turn Lever animation
    
   
    if ( i3 > 0)
    
    {
	    i2= 0.5;
	    i2l = i2l - 0.05;
	    if (i2l < 0) i2l= 0;
	    
	    i2r = i2r + 0.05;
	    if (i2r > 1) i2r= 1;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    if ( i3 == 0 && i2l < 0.5)
    
    {
	    i2l = i2l + 0.05;
	    if (i2l > 0.5) i2l= 0.5;
	    
	    i2r = i2r - 0.05;
	    if (i2r < 0.5) i2r= 0.5;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    
    if ( i3 < 0)
    
    {
	    i2= 0.5;
	    i2l = i2l + 0.05;
	    if (i2l > 1) i2l= 1;
	    
	    i2r = i2r - 0.05;
	    if (i2r < 0) i2r= 0;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
    if ( i3 == 0 && i2l > 0.5)
    
    {
	    i2l = i2l - 0.05;
	    if (i2l < 0.5) i2l= 0.5;
	    
	    i2r = i2r + 0.05;
	    if (i2r > 0.5) i2r= 0.5;
	    
	    SetAnimation (anim_leverleft, i2l);
	    SetAnimation (anim_leverright, i2r);
	    
    }
    
 
     if (antenna_status >= antenna_RAISING) {
		double da = simdt * LIFT_SPEED;
		if (antenna_status == antenna_RAISING) {
			if (antenna_proc > 0.0) antenna_proc = max (0.0, antenna_proc-da);
			else                antenna_status = antenna_UP;
		} else {
			if (antenna_proc < 1.0) antenna_proc = min (1.0, antenna_proc+da);
			else                antenna_status = antenna_DOWN;
		}
		SetAnimation (anim_antenna, antenna_proc);
} 
	








	

//------------------------------------------------------------





}








void BUGGYUMMU::DefineAnimations(void)

{

static UINT LeGrp3[1] = {25};
static MGROUP_ROTATE antenna (0,LeGrp3, 1, _V(-.532,.23,-1.038), _V(1,0,0), (float)(-38*RAD));
anim_antenna = CreateAnimation (0);
      
   AddAnimationComponent  (anim_antenna, 0, 1, &antenna);
 // Levers
   static UINT LeGrp2[2] = {41,63};
   static MGROUP_ROTATE lever2 (0,LeGrp2, 2, _V(-.105,.111,.621), _V(1,0,0), (float)(30*RAD));
   static UINT LeGrp1[2] = {40,62};
   static MGROUP_ROTATE lever1 (0,LeGrp1, 2, _V(.105,.111,.621), _V(1,0,0), (float)(30*RAD));
   
   
      
      
   anim_leverleft = CreateAnimation (0);
      
   AddAnimationComponent  (anim_leverleft, 0, 1, &lever1);
   
   anim_leverright = CreateAnimation (0);
   
   AddAnimationComponent  (anim_leverright, 0, 1, &lever2);   
   
   anim_leverleft1 = CreateAnimation (0);
      

   	
	

 //wheelLGEAR
 static UINT WGrp1[1] = {53};
   static MGROUP_ROTATE wheel1 (0,WGrp1, 1, _V(-5,-.576,.399), _V(1,0,0), (float)(t1*360*RAD));
   //wheelL1
 static UINT WGrp2[1] = {51};
   static MGROUP_ROTATE wheel2 (0,WGrp2, 1, _V(-5,-.576,-.157), _V(1,0,0), (float)(t1*360*RAD));
      //wheelL2
 static UINT WGrp3[1] = {49};
   static MGROUP_ROTATE wheel3 (0,WGrp3, 1, _V(-5,-.576,-.701), _V(1,0,0), (float)(t1*360*RAD));      

  //wheelRGEAR 
   static UINT WGrp9[1] = {43};
   static MGROUP_ROTATE wheel9 (0,WGrp9, 1, _V(5,-.576,.399), _V(1,0,0), (float)(t1*360*RAD));
   //wheelR1
 static UINT WGrp10[1] = {45};
   static MGROUP_ROTATE wheel10 (0,WGrp10, 1, _V(5,-.576,-.157), _V(1,0,0), (float)(t1*360*RAD));
      //wheelR2
 static UINT WGrp11[1] = {47};
   static MGROUP_ROTATE wheel11 (0,WGrp11, 1, _V(5,-.576,-.701), _V(1,0,0), (float)(t1*360*RAD));     

 
   //GEARL1
   static UINT WGrp17[1] = {53};
   static MGROUP_ROTATE wheel17 (0,WGrp17, 1, _V(5,-.576,.399), _V(1,0,0), (float)(-360*RAD));
     //wheelL1
 static UINT WGrp18[1] = {51};
   static MGROUP_ROTATE wheel18 (0,WGrp18, 1, _V(-5,-.576,-.157), _V(1,0,0), (float)(-360*RAD));
      //wheelL2
 static UINT WGrp19[1] = {49};
   static MGROUP_ROTATE wheel19 (0,WGrp19, 1, _V(-5,-.576,-.701), _V(1,0,0), (float)(-360*RAD));      
 
   //GEARRIGHT
   static UINT WGrp25[1] = {43};
   static MGROUP_ROTATE wheel25 (0,WGrp25, 1, _V(-5,-.576,.399), _V(1,0,0), (float)(-360*RAD));
       //wheelR1
 static UINT WGrp26[1] = {45};
   static MGROUP_ROTATE wheel26 (0,WGrp26, 1, _V(-5,-.576,-.157), _V(1,0,0), (float)(-360*RAD));
      //wheelR2
 static UINT WGrp27[1] = {47};
   static MGROUP_ROTATE wheel27 (0,WGrp27, 1, _V(-5,-.576,-.701), _V(1,0,0), (float)(-360*RAD));     

   
    anim_wheelright = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel9);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel10);
   AddAnimationComponent  (anim_wheelright, 0, 1, &wheel11);
   
 anim_wheelleft = CreateAnimation (0);
    
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel1);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel2);
   AddAnimationComponent  (anim_wheelleft, 0, 1, &wheel3);
   
   
   anim_wheelrightrev = CreateAnimation (0);
   
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel25);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel26);
   AddAnimationComponent  (anim_wheelrightrev, 0, 1, &wheel27);
  
   
   
   anim_wheelleftrev = CreateAnimation (0);
    
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel17);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel18);
   AddAnimationComponent  (anim_wheelleftrev, 0, 1, &wheel19);
      
         
  
   }

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
return new BUGGYUMMU (hvessel, flightmodel);
}

DLLCLBK void ovcExit (VESSEL *vessel)
{
if (vessel) delete (BUGGYUMMU*)vessel;
}

// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int BUGGYUMMU::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	// only process keydown events
	if (!down) 
		return 0; 

	//---------------------------------------------------------------------------
	// Ummu Key "E" perform the EVA of the selected member
	if(key==OAPI_KEY_E)
	{
		// PERFORM THE EVA
		
		int Returned=EvaCrewMemberWithStandardSpacesuit(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));
		
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf(SendFocusScreenMessage(),"Transfer to docked ship Ok - %s transfered",Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case EVA_OK:
			sprintf(SendFocusScreenMessage(),"EVA OK - %s left the ship",Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy(SendFocusScreenMessage(),"Error, no one on board, unable to EVA");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy(SendFocusScreenMessage(),"Error, airlock is closed, unable to EVA");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy(SendFocusScreenMessage(),"Error, docked ship's airlock is closed, unable to transfer");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy(SendFocusScreenMessage(),"Error, docked ship is already full transfer failed");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy(SendFocusScreenMessage(),"Error, no crew by this name in ship");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy(SendFocusScreenMessage(),"Error, docked ship do not use UMmu 1.5, ask author to add it See http://orbiter.dansteph.com");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy(SendFocusScreenMessage(),"Misc error with UMMU See http://orbiter.dansteph.com to install it again");
			break;
		}
		return TRUE;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "1" Select next member This is just internal to the demo
	if(key==OAPI_KEY_1)
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendFocusScreenMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}

		// we test that we select existing member
		if(SelectedUmmuMember<Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendFocusScreenMessage(),"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA \"ESC\" to eject",SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	//---------------------------------------------------------------------------
	// Ummu Key "2" Select previous member This is just internal to the demo
	if(key==OAPI_KEY_2)
	{
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber()==0)
		{
			strcpy(SendFocusScreenMessage(),"Sorry no one aboard unable to select");	
			return 1;
		}
		if(SelectedUmmuMember>0)
			SelectedUmmuMember--;
		char * Name=Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf(SendFocusScreenMessage(),"Slot %i %s \"%s\" aged %i Selected for EVA or Transfer, please press \"E\" to EVA \"ESC\" to eject",SelectedUmmuMember,Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),Name,Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}


	//---------------------------------------------------------------------------
	// Ummu Key "A" Switch the airlock door on/off
	if(key==OAPI_KEY_D)
	{
	//RevertRDOOR();
		// switch state
		Crew.SetAirlockDoorState(!Crew.GetAirlockDoorState());
	 //display state
		if(Crew.GetAirlockDoorState()==TRUE)
			strcpy(SendFocusScreenMessage(),"Airlock is now open");	
		else
strcpy(SendFocusScreenMessage(),"Airlock is now closed");	
		return 1;
	}

	//---------------------------------------------------------------------------
	// Eject the guy
	if(key==OAPI_KEY_ESCAPE)
	{
		if(Crew.EjectCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember))==EVA_OK)
			sprintf(SendFocusScreenMessage(),"%s EJECTED",Crew.GetLastEvaedCrewName());SelectedUmmuMember=0;
		return 1;
	}

	//---------------------------------------------------------------------------
	// Use a different Mesh (type "C" then EVA someone)
	if(key==OAPI_KEY_C)
	{
		Crew.SetAlternateMeshToUseForEVASpacesuit("mmu");	// the stock mmu of orbiter located in "meshes/mmu.msh"
		strcpy(SendFocusScreenMessage(),"Mesh changed");
		return 1;
	}

	//---------------------------------------------------------------------------
	// Get some infos Name of ship and total soul aboard
	if(key==OAPI_KEY_S)
	{
		sprintf(SendFocusScreenMessage(),"%i souls aboard ship %s, %i seats available",Crew.GetCrewTotalNumber(),GetName(),1-Crew.GetCrewTotalNumber());
		return 1;
	}

     if(key==OAPI_KEY_G)
	{// hatch
               RevertANTENNA();
               return 1;
	}  
		if(key==OAPI_KEY_N)
	{
		sprintf(SendFocusScreenMessage(),"");
		return 1;
	}
		
		return 0;
}

// ====================================================================
// clbkVisualCreated used to display UMMU initialisation message 
// because oapiDebugString() doesn't work in clbkSetClassCap
// ====================================================================
void BUGGYUMMU::clbkVisualCreated (VISHANDLE vis, int refcount)
{   
	MainExternalMeshVisual = GetMesh(vis,0);
	// We warn the user, in the case UMmu isn't installed it's a good idea to send a "oapiDebugString"
	// Orbiter message, otherwise your addon will not be fully operational and user may not notice. 
	//(nobody read doc, you may be the only one, if you read this send me a postcard ;)
	if(UmmuSdKInitReturnCode==-999)
		strcpy(SendFocusScreenMessage(),"This addon require UMMU addon wich is not installed or outdated. See http://orbiter.dansteph.com to install it");
	else if(UmmuSdKInitReturnCode!=1)
		strcpy(SendFocusScreenMessage(),"Misc error with UMMU See http://orbiter.dansteph.com to install it again");
	// just for the demo
	else if(UmmuSdKInitReturnCode==1)
		strcpy(SendFocusScreenMessage(),"");
}
// ==============================================================
// Visual destroyed
// ==============================================================
void BUGGYUMMU::clbkVisualDestroyed (VISHANDLE vis, int refcount)
{
	MainExternalMeshVisual = 0;	
}









// ========================================================================
// clbkLoadState function of Orbiter - Save scenario when user exit Orbiter 
// ========================================================================
void BUGGYUMMU::clbkSaveState(FILEHANDLE scn)
{

	// ORBITER, default vessel parameters
	SaveDefaultState (scn);

	// UMmu, this will save all our members of crew in scenario
	// as airlock state depend of your ship (you may have a real animated airlock)
	// it's your responsabilites to save and reload UMmu's airlock state.
	Crew.SaveAllMembersInOrbiterScenarios(scn);
}
// ========================================================================
// clbkLoadStateEx function of Orbiter - Load Orbiter scenario 
// ========================================================================
void BUGGYUMMU::clbkLoadStateEx (FILEHANDLE scn, void *status)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) 
	{

		// UMMU, Load all saved member from scenario. 
		// Return TRUE when a "UMMUCREW" line is encountered 
		// FALSE when it must pass this line to you or default parser
		if(Crew.LoadAllMembersFromOrbiterScenario(line)==TRUE)
			continue;

		// ORBITER, unrecognised option - pass to Orbiter's generic parser
		ParseScenarioLineEx (line, status);
	}

}
//------------------------------------------------------------------------------------------------------------------
// This below is not mandatory for UMmu to run.
//------------------------------------------------------------------------------------------------------------------

// ==========================================================
// USING TEXT INTERFACE FOR CREW MANAGMENT (not a good idea)
//
// This is the custom functions that send oapiDebugString() Message on screen
// it simply avoid that the message is sent if our ship don't have the focus
// and ensure that message will be reminded when we focus back on ship
// the Orbiter function below clbkFocusChanged is used to erase message when
// we lost focus.
// It's a bad idea to do an UMmu user interface only by using Debug screen message 
// but if you can't do better you may use this system.
///////////////////////////////////////////////////////////////////////////////////////////
char *BUGGYUMMU::SendFocusScreenMessage()
{
	return cOurMessage;
}
void BUGGYUMMU::clbkFocusChanged(bool getfocus,OBJHANDLE hNewVessel,OBJHANDLE hOldVessel)
{
	// erase message if we focus to another vessel
	if(!getfocus)
	{
		strcpy(oapiDebugString(),"");
	}
}
// ================================================================
// HUD (I put the anim state on the hud because in Poststep
// it forbids ummu messages to be displayed).
// ================================================================


// ==========================================================================================
// USING STANDARD FUNCTION ID,NAME AND SPACESUIT (Good idea)
// **TEST: replace "Crew.EvaCrewMember" with this function in clbkConsumeBufferedKey (key E)**
// ==========================================================================================
int BUGGYUMMU::EvaCrewMemberWithStandardSpacesuit(char* CrewName)
{
	char TempMiscID[5]={0};
	int I=0;
	char MeshToUse[26]={"ummu/ummu"}; // default mesh if not found
	// retriew the crew's MiscID
	strcpy(TempMiscID,Crew.GetCrewMiscIdByName(CrewName));
	// search if we find this ID in the list
	for(I=0;I<NUMBER_OF_MMU_TYPE;I++)
	{
		if(strcmp(UmmuMiscID[I],TempMiscID)==0)
		{
			// found ! copy the mesh filename to use for EVA
			strcpy(MeshToUse,UmmuMeshUsed[I]);
			break;
		}
	}
	// Set the mesh filename to use
	Crew.SetAlternateMeshToUseForEVASpacesuit(MeshToUse);
	// Eva the crew member with this mesh
	return Crew.EvaCrewMember(CrewName);
}
void BUGGYUMMU::RevertANTENNA (void)
{
	antenna_status = ((antenna_status == antenna_UP || antenna_status == antenna_RAISING) ?
		antenna_LOWERING : antenna_RAISING);
}