// ==============================================================
//                 ORBITER MODULE: NASABUGGY
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2004 Martin Schweiger
//                   All rights reserved
//
// NASABUGGY.cpp
// Control module for NASABUGGY vessel class
//
// Notes:
// This is an example for a "minimal" vessel implementation which
// only overloads the clbkSetClassCaps method to define vessel
// capabilities and otherwise uses the default VESSEL class
// behaviour.
// ==============================================================



#include "orbitersdk.h"
#include "UMmuSDK.h"

// ==============================================================
// Some vessel parameters
// ==============================================================
const double FUELMASS = 10;
const double ISP = 11e12;
const double MAXMAINTH = 500;
const double MAXRETROTH = 500;
const double MAXRCSTH = 750;

// Interface for derived vessel class: QJBStar
// ==========================================================

class NASABUGGY: public VESSEL2 {
public:
    NASABUGGY (OBJHANDLE hObj, int fmodel);
	void clbkSaveState(FILEHANDLE scn);
       void clbkPostCreation (void);
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
 	void DefineAnimations(void); 
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkPostStep (double simtt, double simdt, double mjd);
	void clbkFocusChanged(bool getfocus,OBJHANDLE hNewVessel,OBJHANDLE hOldVessel);
enum antennaStatus { antenna_UP, antenna_DOWN, antenna_RAISING, antenna_LOWERING } antenna_status;
	void RevertANTENNA (void);
// UMMU 1.5 DECLARATION
	UMMUCREWMANAGMENT Crew;
	void clbkSetClassCaps_UMMu(void);	// our special SetClassCapfunction added for more readability
	int SelectedUmmuMember;				// for the SDK demo, select the member to eva
	int UmmuSdKInitReturnCode;			// to display init message to user (in case Ummu isn't installed on Orbiter)

	// our custom oapiDebugString() function (an orbiter function that display debug text at bottom of screen)
	// Our custom function display only message if we have focus. The variables below is used by this function.
	// don't forget if you use such text interface in your vessel that you must initialise the variables
	// in set class cap and do also some code in clbkFocusChanged
	char cOurMessage[1000];
	char *SendFocusScreenMessage();	

	// If you want to use standard ID function see bottom of UMmuExample.cpp
	int EvaCrewMemberWithStandardSpacesuit(char* CrewName);	
private:



THRUSTER_HANDLE th_main, th_retro,  th_rcs[4], th_group[2] ;
UINT BUG0;
UINT BUG1;
UINT BUG2;
UINT BUG3;
UINT BUG4;
UINT BUG5;
UINT BUG6;
UINT BUG7;
UINT BUG8;

UINT anim_wheel;
UINT anim_wheelleft;
UINT anim_wheelright;
UINT anim_wheelleftrev;
UINT anim_wheelrightrev;
UINT anim_leverleft;
UINT anim_leverright;
UINT anim_leverleft1;
UINT anim_leverright1;
UINT anim_antenna;
int crewaboard;
int NASABUGGYSoundID;
double LIFT_SPEED;
double L_SPEED;
double ANT_OPERATING_SPEED;
double antenna_proc;
double db;
double i;
double i3;
double t1;
double main_on;
double retro_on;
double i2l;
double i2r;
double i2;

};